import html
import urllib.parse
from typing import Dict

import lxml.html  # nosec  # noqa DUO107
import sphinx.util
from sphinx.application import Sphinx

from .utils import Obfuscator

logger = sphinx.util.logging.getLogger(__name__)


def html_page_context_handler(
    app: Sphinx, pagename: str, templatename: str, context: Dict, doctree: bool
):
    """Search html for 'mailto' links and obfuscate them"""
    if not app.config["email_automode"]:
        return
    if not doctree:
        return

    tree = lxml.html.fragment_fromstring(context["body"], create_parent="body")
    links = tree.iterlinks()
    links = filter(lambda link: link[2].startswith("mailto:"), list(links))

    for link in links:
        old_node = link[0]
        old_node_str = lxml.html.tostring(old_node, encoding="unicode", with_tail=False)
        tail_str = old_node.tail

        obfuscated = Obfuscator().js_obfuscated_text(old_node_str)
        new_node = lxml.html.fragment_fromstring(obfuscated)
        new_node.tail = tail_str

        old_node.getparent().replace(old_node, new_node)

    child_strs = map(
        lambda child: lxml.html.tostring(child, pretty_print=True, encoding="unicode"),
        tree.iterchildren(),
    )
    context["body"] = "".join(child_strs)



from typing import Any, Dict

import sphinx.util
from sphinx.application import Sphinx

from .handlers import html_page_context_handler
from .roles import EmailRole

try:
    from importlib import metadata
except ImportError:
    import importlib_metadata as metadata

try:
    __version__ = metadata.version("sphinxcontrib-email")
except metadata.PackageNotFoundError:
    pass

logger = sphinx.util.logging.getLogger(__name__)


def setup(app: Sphinx) -> Dict[str, Any]:
    app.add_config_value(name="email_automode", default=False, rebuild="env")
    app.connect("html-page-context", html_page_context_handler)
    app.add_role("email", EmailRole())
    return {"version": ".".join(__version__.split(".")[:3])}



import re
import textwrap
import xml.sax.saxutils  # nosec
from xml.etree import ElementTree as ET  # nosec  # noqa DUO107

import sphinx.util

logger = sphinx.util.logging.getLogger(__name__)


class Obfuscator:
    def __init__(self):
        self.rot_13_trans = str.maketrans(
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
            "NOPQRSTUVWXYZABCDEFGHIJKLMnopqrstuvwxyzabcdefghijklm",
        )

    def rot_13_encrypt(self, line: str) -> str:
        """Rotate 13 encryption."""
        line = line.translate(self.rot_13_trans)
        line = re.sub(r"(?=[\"])", r"\\", line)
        line = re.sub("\n", r"\n", line)
        line = re.sub(r"@", r"\\100", line)
        line = re.sub(r"\.", r"\\056", line)
        line = re.sub(r"/", r"\\057", line)
        return line

    def xml_to_unesc_string(self, node: ET.Element) -> str:
        """Return unescaped xml string"""
        text = xml.sax.saxutils.unescape(
            ET.tostring(node, encoding="unicode", method="html"),
            {"&apos;": "'", "&quot;": '"'},
        )
        return text

    def js_obfuscated_text(self, text: str) -> str:
        """ROT 13 encryption with embedded in Javascript code to decrypt in the
        browser.
        """
        xml_node = ET.Element("script")
        xml_node.attrib["type"] = "text/javascript"
        js_script = textwrap.dedent(
            """\
            document.write(
                "{text}".replace(/[a-zA-Z]/g,
                    function(c){{
                        return String.fromCharCode(
                            (c<="Z"?90:122)>=(c=c.charCodeAt(0)+13)?c:c-26
                        );
                    }}
                )
            );"""
        )
        xml_node.text = js_script.format(text=self.rot_13_encrypt(text))

        return self.xml_to_unesc_string(xml_node)

    def js_obfuscated_mailto(self, email: str, displayname: str = None) -> str:
        """ROT 13 encryption within an Anchor tag w/ a mailto: attribute"""
        xml_node = ET.Element("a")
        xml_node.attrib["class"] = "reference external"
        xml_node.attrib["href"] = f"mailto:{email}"
        xml_node.text = displayname or email

        return self.js_obfuscated_text(self.xml_to_unesc_string(xml_node))



import re
from typing import List, Tuple

import sphinx.util
from docutils import nodes
from docutils.nodes import Element, Node, system_message
from docutils.parsers.rst.states import Inliner
from sphinx.util.docutils import SphinxRole

from .utils import Obfuscator

logger = sphinx.util.logging.getLogger(__name__)


class EmailRole(SphinxRole):
    def run(self) -> Tuple[List[Node], List[system_message]]:
        """Role to obfuscate e-mail addresses.

        Handle addresses of the form
        "name@domain.org"
        "Name Surname <name@domain.org>"
        """
        pattern = (
            r"^(?:(?P<name>.*?)\s*<)?(?P<email>\b[-.\w]+@[-.\w]+\.[a-z]{2,4}\b)>?$"
        )
        match = re.search(pattern, self.text)
        if not match:
            return [], []
        data = match.groupdict()

        obfuscated = Obfuscator().js_obfuscated_mailto(
            email=data["email"], displayname=data["name"]
        )
        node = nodes.raw("", obfuscated, format="html")
        return [node], []
